module scomp(clock, reset, memory_address, memory_data_out, 
	register_A, pc, write_enable);
input clock, reset;
output [15:0] memory_data_out;
output [15:0] register_A;
output [7:0] pc;
output [7:0] memory_address;
//output [2:0] state;
reg [2:0] state;
reg [15:0] register_A;
reg [7:0] pc;
reg [1:0] control;

reg [7:0] memory_address;
reg [7:0] data_address;
wire [15:0] alu_out;
output write_enable;

wire [7:0] opcode;
assign opcode = memory_data_out[15:8];

parameter
	reset_pc = 3'd0,
	decode   = 3'd1,
	execute = 3'd2,
	store1 = 3'd3,
	store2 = 3'd4;

// instantiate memory (256 16-bit words)
mem mem_component(
	.data (register_A),
	.address (memory_address),
	.wren (write_enable),
	.q (memory_data_out),
	.inclock (clock)
	);

// instantiate ALU
ALU alu_component(
	.A(register_A),
	.B(memory_data_out),
	.C(alu_out),
	.control(control)
	);
	
assign write_enable = (state==store1);
	
always
case (state)
reset_pc: memory_address <= 8'h00;
decode:  memory_address <= memory_data_out[7:0];
execute: memory_address <= pc;
store1:  memory_address <= data_address;
store2:  memory_address <= pc;
default: memory_address <= 8'hff;
endcase

// finite state machine
always @(posedge clock, negedge reset)
	begin
	if (~reset) state<=reset_pc;
	else
	case (state)
	reset_pc :
		begin
		pc <= 8'h00;
		register_A <= 16'h0000;
		state <= decode;
		end
	// decode opcode, increment program counter
	decode :
		begin
		if (opcode==8'h05)
			pc <= memory_data_out[7:0];
		else if (opcode==8'h06 && register_A[15])
			pc <= memory_data_out[7:0];
		else
			pc <= pc + 8'h01;
		data_address <= memory_data_out[7:0];
		// decode opcode
		case (opcode)
		8'h01 : // load
			begin
			control <= 2'b01;
			state <= execute;
			end
		8'h02 : // add
			begin
			control <= 2'b10;
			state <= execute;
			end
		8'h03 : // subtract
			begin
			control <= 2'b11;
			state <= execute;
			end
		8'h04 : // store
			begin
			control <= 2'b00;
			state <= store1;
			end
		default:
			begin
			control <= 2'b00;
			state <= decode;
			end
		endcase
		end
	// execute instruction, setup instruction address
	// Add opcode
	execute :
		begin
		register_A <= alu_out;
		state <= decode;
		end
	store1 :
		begin
		state <= store2;
		end
	// default state transition
	default : state <= decode;
	endcase
	end
endmodule